from . import _init_commands_v1

# autogenerated by 'robotpy-build create-imports wpilib.command wpilib.command._commands_v1'
from ._commands_v1 import (
    Button,
    ButtonScheduler,
    CancelButtonScheduler,
    Command,
    CommandGroup,
    CommandGroupEntry,
    ConditionalCommand,
    HeldButtonScheduler,
    InstantCommand,
    InternalButton,
    JoystickButton,
    NetworkButton,
    PIDCommand,
    PIDSubsystem,
    POVButton,
    PressedButtonScheduler,
    PrintCommand,
    ReleasedButtonScheduler,
    Scheduler,
    StartCommand,
    Subsystem,
    TimedCommand,
    ToggleButtonScheduler,
    Trigger,
    WaitCommand,
    WaitForChildren,
    WaitUntilCommand,
)

__all__ = [
    "Button",
    "ButtonScheduler",
    "CancelButtonScheduler",
    "Command",
    "CommandGroup",
    "CommandGroupEntry",
    "ConditionalCommand",
    "HeldButtonScheduler",
    "InstantCommand",
    "InternalButton",
    "JoystickButton",
    "NetworkButton",
    "PIDCommand",
    "PIDSubsystem",
    "POVButton",
    "PressedButtonScheduler",
    "PrintCommand",
    "ReleasedButtonScheduler",
    "Scheduler",
    "StartCommand",
    "Subsystem",
    "TimedCommand",
    "ToggleButtonScheduler",
    "Trigger",
    "WaitCommand",
    "WaitForChildren",
    "WaitUntilCommand",
]

del _init_commands_v1

try:
    from .version import version as __version__
except ImportError:
    __version__ = "master"
